@@ -10,14 +10,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import urllib
+from urllib.parse import urlparse
+
 try:
     import simplejson
     JSONDecodeError = simplejson.scanner.JSONDecodeError
 except ImportError:
     JSONDecodeError = ValueError
 import iso8601
-import urllib
-
 from keystoneauth1 import adapter
 
 from openstack import _log
@@ -213,12 +214,12 @@ class Proxy(adapter.Adapter):
         key = '.'.join(
             [self._statsd_prefix, self.service_type, method]
             + name_parts)
-        if response is not None:
-            duration = int(response.elapsed.total_seconds() * 1000)
-            self._statsd_client.timing(key, duration)
-            self._statsd_client.incr(key)
-        elif exc is not None:
-            self._statsd_client.incr('%s.failed' % key)
+        with self._statsd_client.pipeline() as pipe:
+            if response is not None:
+                pipe.timing(key, response.elapsed)
+                pipe.incr(key)
+            elif exc is not None:
+                pipe.incr('%s.failed' % key)
 
     def _report_stats_prometheus(self, response, url=None, method=None,
                                  exc=None):
@@ -226,10 +227,13 @@ class Proxy(adapter.Adapter):
             url = response.request.url
         if response is not None and not method:
             method = response.request.method
+        parsed_url = urlparse(url)
+        endpoint = "{}://{}{}".format(
+            parsed_url.scheme, parsed_url.netloc, parsed_url.path)
         if response is not None:
             labels = dict(
                 method=method,
-                endpoint=url,
+                endpoint=endpoint,
                 service_type=self.service_type,
                 status_code=response.status_code,
             )
@@ -512,7 +516,7 @@ class Proxy(adapter.Adapter):
               paginated=True, base_path=None, **attrs):
         """List a resource
 
-        :param resource_type: The type of resource to delete. This should
+        :param resource_type: The type of resource to list. This should
                               be a :class:`~openstack.resource.Resource`
                               subclass with a ``from_id`` method.
         :param value: The resource to list. It can be the ID of a resource, or
