@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.identity.v3 import endpoint
 
@@ -26,20 +26,27 @@ EXAMPLE = {
 }
 
 
-class TestEndpoint(testtools.TestCase):
+class TestEndpoint(base.TestCase):
 
     def test_basic(self):
         sot = endpoint.Endpoint()
         self.assertEqual('endpoint', sot.resource_key)
         self.assertEqual('endpoints', sot.resources_key)
         self.assertEqual('/endpoints', sot.base_path)
-        self.assertEqual('identity', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
-        self.assertTrue(sot.patch_update)
+        self.assertEqual('PATCH', sot.commit_method)
+        self.assertDictEqual(
+            {
+                'interface': 'interface',
+                'service_id': 'service_id',
+                'limit': 'limit',
+                'marker': 'marker',
+            },
+            sot._query_mapping._mapping)
 
     def test_make_it(self):
         sot = endpoint.Endpoint(**EXAMPLE)
