@@ -213,3 +213,21 @@ class TestRaiseFromResponse(base.TestCase):
         self.assertEqual(response.status_code, exc.status_code)
         self.assertEqual(self.message, exc.details)
         self.assertIn(self.message, str(exc))
+
+    def test_raise_wsme_format(self):
+        response = mock.Mock()
+        response.status_code = 404
+        response.headers = {
+            'content-type': 'application/json',
+        }
+        response.json.return_value = {
+            'faultstring': self.message,
+            'faultcode': 'Client',
+            'debuginfo': None,
+        }
+        exc = self.assertRaises(exceptions.NotFoundException,
+                                self._do_raise, response,
+                                error_message=self.message)
+        self.assertEqual(response.status_code, exc.status_code)
+        self.assertEqual(self.message, exc.details)
+        self.assertIn(self.message, str(exc))
